'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Diagnostics
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Windows.Forms
Imports System.Windows.Forms.Design
Imports System.Windows.Forms.Design.Behavior

Namespace Extensions

    ''' <summary>
    ''' A default implementation of a <see cref="Glyph"/> for the
    ''' <see cref="DesignSurfaceExtenderLibrary"/>.
    ''' </summary>
    Public MustInherit Class ExtensionGlyph
        Inherits Glyph
        Implements IDisposable

        Private disposed As Boolean = False

        ''' <summary>
        ''' Caches a reference to the <see cref="BehaviorService"/> for convenience.  
        ''' </summary>
        Protected behaviorService As BehaviorService

        ''' <summary>
        ''' Caches a reference to the <see cref="IComponentChangeService"/> for convenience.  
        ''' </summary>
        Protected changeService As IComponentChangeService

        ''' <summary>
        ''' Caches a reference to the <see cref="ISelectionService"/> for convenience.  
        ''' </summary>
        Protected selectionService As ISelectionService

        ''' <summary>
        ''' Caches a reference to the <see cref="IReferenceService"/> for convenience.  
        ''' </summary>
        Protected referenceService As IReferenceService

        ''' <summary>
        ''' Caches a reference to an <see cref="IServiceProvider"/> for convenience.  
        ''' </summary>
        Protected serviceProvider As IServiceProvider

        ''' <summary>
        ''' Backs the <see cref="RelatedControl"/> property.
        ''' </summary>
        Protected relatedControlValue As Control

        ''' <summary>
        ''' Caches the <see cref="Adorner"/> which hosts this <see cref="Glyph"/>.
        ''' </summary>
        Protected adorner As Adorner

        ''' <summary>
        ''' Backs the <see cref="DependentProperties"/> property.
        ''' </summary>
        Protected dependentPropertiesValue As ReadOnlyCollection(Of String)

        ''' <summary>
        ''' Backs the <see cref="Bounds"/> property.
        ''' </summary>
        Protected boundsValue As Rectangle

        ''' <summary>
        ''' Initializes a new instance of the <see cref="ExtensionGlyph"/> class.
        ''' </summary>
        ''' <param name="c">The control to be assigned as the <see cref="RelatedControl"/>. 
        ''' Can be null.</param>
        ''' <param name="behaviorService">A reference to the <see cref="BehaviorService"/>.</param>
        ''' <param name="changeService">A reference to the <see cref="IComponentChangeService"/>.</param>
        ''' <param name="selectionService">A reference to the <see cref="ISelectionService"/>.</param>
        ''' <param name="referenceService">A reference to the <see cref="IReferenceService"/>.</param>
        ''' <param name="serviceProvider">A reference to an <see cref="IServiceProvider"/>.</param>
        ''' <param name="relatedBehavior">The <see cref="Behavior"/> which controls this <see cref="Glyph"/>.</param>
        ''' <param name="adorner">The <see cref="Adorner"/> which hosts this <see cref="Glyph"/>.</param>
        Protected Sub New( _
        ByVal c As Control, _
        ByVal behaviorService As BehaviorService, _
        ByVal changeService As IComponentChangeService, _
        ByVal selectionService As ISelectionService, _
        ByVal referenceService As IReferenceService, _
        ByVal serviceProvider As IServiceProvider, _
        ByVal relatedBehavior As Behavior, _
        ByVal adorner As Adorner)
            MyBase.New(relatedBehavior)

            Me.CheckParameters( _
                behaviorService, _
                changeService, _
                selectionService, _
                referenceService, _
                serviceProvider, _
                relatedBehavior, _
                adorner)

            Me.relatedControlValue = c
            Me.behaviorService = behaviorService
            Me.changeService = changeService
            Me.selectionService = selectionService
            Me.referenceService = referenceService
            Me.serviceProvider = serviceProvider
            Me.adorner = adorner

            Me.ComputeGeometry()

            AddHandler Me.changeService.ComponentChanged, AddressOf ComponentChangedEventHandler
        End Sub

        ''' <summary>
        ''' Validates inputs to the <see cref="ExtensionGlyph"/> constructor.
        ''' </summary>
        ''' <param name="behaviorService">A reference to the <see cref="BehaviorService"/>.</param>
        ''' <param name="changeService">A reference to the <see cref="IComponentChangeService"/>.</param>
        ''' <param name="selectionService">A reference to the <see cref="ISelectionService"/>.</param>
        ''' <param name="referenceService">A reference to the <see cref="IReferenceService"/>.</param>
        ''' <param name="serviceProvider">A reference to an <see cref="IServiceProvider"/>.</param>
        ''' <param name="relatedBehavior">The <see cref="Behavior"/> which controls this <see cref="Glyph"/>.</param>
        ''' <param name="adorner">The <see cref="Adorner"/> which hosts this <see cref="Glyph"/>.</param>
        Protected Overridable Sub CheckParameters( _
        ByVal behaviorService As BehaviorService, _
        ByVal changeService As IComponentChangeService, _
        ByVal selectionService As ISelectionService, _
        ByVal referenceService As IReferenceService, _
        ByVal serviceProvider As IServiceProvider, _
        ByVal relatedBehavior As Behavior, _
        ByVal adorner As Adorner)

            If (behaviorService Is Nothing) Or _
                (changeService Is Nothing) Or _
                (selectionService Is Nothing) Or _
                (referenceService Is Nothing) Or _
                (serviceProvider Is Nothing) Or _
                (relatedBehavior Is Nothing) Or _
                (adorner Is Nothing) Then

                Throw New ArgumentNullException()
            End If
        End Sub

        ''' <summary>
        ''' Calculataes the geometry of the <see cref="ExtensionGlyph"/>.
        ''' </summary>
        ''' <remarks>Deriving classes must override this method to 
        ''' calculate the <see cref="Bounds"/>, <see cref="Control.Location"/>, and
        ''' similar properties.
        ''' </remarks>
        Protected MustOverride Sub ComputeGeometry()

        ''' <summary>
        ''' Defines the collection of properties on which 
        ''' the <see cref="ExtensionGlyph"/> depends. 
        ''' </summary>
        ''' <returns>A type-dependent collection of strings representing 
        ''' the properties on which the specific <see cref="ExtensionGlyph"/> 
        ''' type depends.</returns>
        Protected MustOverride Function PopulateDependentProperties() As ReadOnlyCollection(Of String)

        ''' <summary>
        ''' Gets a collection of strings representing the properties on which 
        ''' the specific <see cref="ExtensionGlyph"/> type depends.
        ''' </summary>
        Protected ReadOnly Property DependentProperties() As ReadOnlyCollection(Of String)
            Get
                If Me.dependentPropertiesValue Is Nothing Then
                    Me.dependentPropertiesValue = Me.PopulateDependentProperties()
                End If

                Return Me.dependentPropertiesValue
            End Get
        End Property

        ''' <summary>
        ''' Handles the <see cref="IComponentChangeService.ComponentChanged"/> event.
        ''' </summary>
        ''' <param name="sender">The <see cref="IComponent"/> which raied the event </param>
        ''' <param name="e">An <see cref="EventArgs"/> which holds the event data.</param>
        Protected Overridable Sub ComponentChangedEventHandler( _
        ByVal sender As Object, _
        ByVal e As ComponentChangedEventArgs)
            If Object.ReferenceEquals( _
                e.Component, _
                Me.relatedControlValue) Then
                Dim propertyName As String = e.Member.Name

                If Me.DependentProperties.Contains(propertyName) Then
                    Me.ComponentChangedCallback(propertyName)
                End If
            End If
        End Sub

        ''' <summary>
        ''' Default <see cref="ExtensionGlyph"/> behavior when the given property
        ''' of the related control changes.
        ''' </summary>
        ''' <param name="propertyName">The name of the property that changed.</param>
        Protected Overridable Sub ComponentChangedCallback(ByVal propertyName As String)
            Me.ComputeGeometry()

            If Me.adorner.Enabled Then
                Me.adorner.Invalidate()
            End If
        End Sub

        ''' <summary>
        ''' Gets the bounds of the <see cref="ExtensionGlyph"/>. 
        ''' </summary>
        Public Overrides ReadOnly Property Bounds() As Rectangle
            Get
                Return Me.boundsValue
            End Get
        End Property

        ''' <summary>
        ''' The control on the design surface to which this <see cref="ExtensionGlyph"/> is bound.
        ''' </summary>
        ''' <remarks>Can be null for <see cref="ExtensionGlyph"/> implementations
        ''' which aren't bound to particular controls.
        ''' </remarks>
        Public ReadOnly Property RelatedControl() As Control
            Get
                Return Me.relatedControlValue
            End Get
        End Property

#Region "IDisposable Implementation"

        ''' <summary>
        ''' Releases all resources used by the <see cref="ExtensionGlyph"/>. 
        ''' </summary>
        ''' <remarks>The <see cref="ExtensionGlyph"/> class implements the 
        ''' the Dispose pattern in order to release event handlers.
        ''' </remarks>
        Public Sub Dispose() Implements IDisposable.Dispose
            ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(True)
            GC.SuppressFinalize(Me)
        End Sub

        ''' <summary>
        ''' Releases the unmanaged resources used by the <see cref="ExtensionGlyph"/> and
        ''' optionally releases the managed resources. 
        ''' </summary>
        ''' <param name="disposing"><c>true</c> to release both managed and unmanaged resources; 
        ''' <c>false</c> to release only unmanaged resources. </param>
        Protected Overridable Sub Dispose(ByVal disposing As Boolean)
            If Not Me.disposed Then
                If disposing Then
                    If Me.changeService IsNot Nothing Then
                        AddHandler Me.changeService.ComponentChanged, AddressOf ComponentChangedEventHandler
                    End If
                End If
            End If
            Me.disposed = True
        End Sub

#End Region

    End Class

End Namespace
